/*////////////////////////////////////////////////////////////////////////
Copyright (c) 1992 Electrotechnical Laboratry (ETL)

Permission to use, copy, modify, and distribute this material 
for any purpose and without fee is hereby granted, provided 
that the above copyright notice and this permission notice 
appear in all copies, and that the name of ETL not be 
used in advertising or publicity pertaining to this 
material without the specific, prior written permission 
of an authorized representative of ETL.
ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY 
OF THIS MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", 
WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES.
/////////////////////////////////////////////////////////////////////////
Content-Type:	program/C; charset=US-ASCII
Program:	xnntps.c (xnntp server library)
Author:		Yutaka Sato <ysato@etl.go.jp>
Description:
History:
 V2.0	930822	added logging
	930822	added XHEAD with NHC
///////////////////////////////////////////////////////////////////////*/
char *xnntp_version = "2.0 (16 December 93)";

#ifndef INCLUDE_DONE
#include "common.h"

FILE *open_article();

extern  int	(*xnntp_rstat)();
extern  int	(*xnntp_rbody)();
#define RSTAT	(*xnntp_rstat)
#define RBODY	(*xnntp_rbody)
#endif

#ifndef XNNTP_LOGFILE
#define XNNTP_LOGFILE "/tmp/xnntp.log"
#endif

static xhelp(){
	RSTAT(INF_HELP,"This server accepts the following commands:");
	RBODY("XLIST [difference]       LIST [only changed] groups.\n");
	RBODY("XHIST {msgid,date-spec}  Retrieve the history file.\n");
	RBODY("XMIME {enable/disable}   Control MIME decoder.\n");
	RBODY("\n");
	RBODY("Send bugs to Yutaka Sato <ysato@etl.go.jp>\n");
	RBODY(0);
}

static int logged;
xnntp(ac,av)
	char *av[];
{	char *com;

	com = av[0];
	if( strcmp(com,"xuser") == 0) xuser(ac,av); else
	if( strcmp(com,"xhist") == 0) xhist(ac,av); else
	if( strcmp(com,"xlist") == 0) xlist(ac,av); else
#ifdef XMIME
	if( strcmp(com,"xmime") == 0) xmime(ac,av); else
#endif
#ifdef NHCLIB
	if( strcmp(com,"xhead") == 0) xhead(ac,av); else
#endif
	if( strcmp(com,"help" ) == 0) xhelp(ac,av); else
	if( strcmp(com,"quit" ) == 0) xquit(ac,av); else xerror(ac,av);

	if( logged == 0 )
		xnntp_login();
}

static xnntp_rstat0(code,form,a,b,c,d,e,f,g){
	printf("%d ",code);
	printf(form,a,b,c,d,e,f,g);
	printf("\r\n",code);

	if( code != OK_HEAD )
		fflush(stdout);
}
static xnntp_rbody0(line){
	if(line)
		fputs(line,stdout);
	else{
		puts(".");
		fflush(stdout);
	}
}

int (*xnntp_rstat)() = xnntp_rstat0;
int (*xnntp_rbody)() = xnntp_rbody0;

/*
 *	user, tty, TERM, host, command, time
 */
static char xutmp[256];
xuser(ac,av)
	char *av[];
{	FILE *lfp;
	int ai;

	lfp = fopen(XNNTP_LOGFILE,"a");
	if( lfp != NULL ){
		RSTAT(OK_AUTH,   "%s","user log added.");
		fclose(lfp);
		for( ai = 1; ai < ac; ai++ ){
			strcat(xutmp,av[ai]);
			strcat(xutmp," ");
		}
	}else	RSTAT(ERR_NOAUTH,"%s","cannot write log.");
}

xnntp_login(){
#ifdef XNNTP_UTMP
	if( xutmp[0] == 0 )
		sprint_xutmp(xutmp,"?");
	add_xutmp(XNNTP_UTMP,"LOGIN", xutmp);
	logged = 1;
#endif
}
xnntp_logout(){
#ifdef XNNTP_UTMP
	add_xutmp(XNNTP_UTMP,"LOGOUT",xutmp);
#endif
}

xnntp_startmsg(){
	char host[1024];

	gethostname(host,sizeof(host));
	RSTAT(OK_NOPOST,"%s XNNTP server version %s ready.",
		host,xnntp_version);
}
static xerror(ac,av)
	char *av[];
{
	RSTAT(ERR_CMDSYN,"Unknown Command: %s %s",av[0],av[1]);
}
static xquit(){
	RSTAT(OK_GOODBYE,"bye.");
	xnntp_logout();
	exit(0);
}

static int previous_mtime;
static char *previous_active;
static xlist(ac,av)
	char *av[];
{	FILE *fp;
	char *active,*line,*prev,linebuff[2048];
	int mtime,actsize,leng;

	if((fp = fopen(ACTIVE_FILE,"r")) == NULL){
		RSTAT(ERR_FAULT,"Cannot open active file.");
		return;
	}

	mtime = FileMtime(fp);
	RSTAT(OK_GROUPS,
		"%d Newsgroups in form \"group high low y/n/m\".",mtime);

	if( ac == 2 && strcmp(av[1],"difference") == 0 ){
		if( mtime != previous_mtime ){
			prev = previous_active;
			actsize = FileSize(fp) + 1;
			active = (char*)malloc(actsize);

			line = active;
			while( 1 < actsize && fgets(line,actsize,fp) != NULL ){
				leng = strlen(line);
				if( prev == 0 || strncmp(line,prev,leng) != 0 )
					RBODY(line);
				if( prev != 0 && (prev = index(prev,'\n')) )
					prev++;
				line += strlen(line);
				actsize -= leng;
			}
			if( previous_active )
				free(previous_active);
			previous_active = active;
			previous_mtime = mtime;
		}
	}else{
		while( fgets(linebuff,sizeof(linebuff),fp) != NULL )
			RBODY(linebuff);
	}
	RBODY(0);
	fclose(fp);
}



#ifdef NHCLIB
static char LastGroup[256];
static int LastAnum;
static int BENCH = 0;


xhead(ac,av)
	char *av[];
{	char head_cache1[0x10000],head_cache2[0x10000],*head = head_cache1;
	char msgid[512];
	int bytes;
	FILE *fp = 0;

if(BENCH){
	bytes = 256;
	head = head_cache1;
	sprintf(head,"%-255s\n","Message-ID: <1234>\n");
	goto OUT;
}
	init_nhc();
	if( ac == 2 ){
		if( index(av[1],':') )
			sscanf(av[1],"%[^:]:%d",LastGroup,&LastAnum);
		else	LastAnum = atoi(av[1]);
	}
	bytes = nhc_get(LastGroup,LastAnum,head_cache1);

	if( 0 < bytes ){
		head = head_cache1;
	}else
	if( bytes < 0 ){
		RSTAT(ERR_FAULT,"it seems to be cancelled.");
		return;
	}else
	if( fp = open_article(LastGroup,LastAnum,head_cache1) ){
/*
		MIME_strHeaderDecode(head_cache1,head_cache2);
		nhc_put(LastGroup,LastAnum,head_cache2);
*/
/*
extern int COOK_MIME_HDR;
if( COOK_MIME_HDR )
	head = head_cache2;
else	head = head_cache1;
*/
		head = head_cache1;
	}else{
		nhc_cancel(LastGroup,LastAnum);
		RSTAT(ERR_FAULT,"it might be cancelled.");
		return;
	}

OUT:
	get_field(head,"Message-ID: ",msgid);
	RSTAT(OK_HEAD,"%d %s Article retrieved: fields follow:",
		LastAnum,msgid);
	RBODY(head);
	RBODY(0);

	if( fp ){
		/* should append
			Arrival-Date:
			Parent-Xref:
		*/
		nhc_fput(LastGroup,LastAnum,fp);
		fclose(fp);
	}
}


static get_field(head,field,buff)
	char *head,*field,*buff;
{	char *hp,hc,*bp;
	int flen;

	*buff = 0;
	flen = strlen(field);

	for( hp = head; hc = *hp; hp++ ){
		if( *field == hc && strncmp(hp,field,flen) == 0 ){
			bp = buff;
			for( hp += flen; hc = *hp; hp++ ){
				if( hc == '\n' || hc == '\r' ){
					*bp = 0;
					break;
				}
				*bp++ = hc;
			}
			return;
		}
		if( (hp = index(hp,'\n')) == 0 )
			break;
	}
}


char *STRING_ALLOC(s){ return strcpy(malloc(strlen(s)+1),s); }
mkdirr_silently(path,mode){ }
cosmos_bugreport(){ exit(1); }
news_active_first(){ return 1; }
dbg_printf(){ }
char *rstrcat(d,s) char *d; { strcpy(d,s); return d+strlen(d); }

int (*CODEC_message)();
int (*NHC_message)();
static nhc_message(fmt,a,b,c,d,e,f,g){
/*
	fprintf(stderr,fmt,a,b,c,d,e,f,g);
	fprintf(stderr,"\n");
*/
}

char *
newshead_cachedir(dir,len) char *dir; { strcpy(dir,XHEAD_CACHE); return dir; }
char *(*NEWSHEAD_CACHEDIR)() = newshead_cachedir;
init_nhc(){
	NHC_message = nhc_message;
	CODEC_message = nhc_message;
}

#endif
